/*
 *  JVending
 *  Copyright (C) 2004  Shane Isbell
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package javax.provisioning;

import java.util.EventObject;

/**
 * The DeliveryEvent class is used to notify the provisioning developer of events that occur during or after delivery
 * of a bundle. The events may originate either from the provisioning adapter used to deliver the bundle, or from
 * the client device to which the adapter is delivering the bundle. The source of the event (see
 * java.util.EventObject.getSource()) is the DeliveryContext associated with the device to which the provisioning
 * server is delivering.
 */

public class DeliveryEvent extends EventObject implements java.io.Serializable {

    public static int COMPLETE = 0;

    public static int CONFIRMED = 1;

    public static int DELETED = 2;

    public static int FAILED = 3;

    public static int INVALID_FULFILLMENT_ID = 0;

    public static int OTHER = 5;

    private final String adapterName;

    private final int code;

    private final DeliveryContext deliveryContext;

    private final String description;

    private final int type;

    private final String fulfillmentID;

    public DeliveryEvent(DeliveryContext deliveryContext, String adapterName, int type,
                         String fulfillmentID, int code, String description) {
        super(deliveryContext);
        this.deliveryContext = deliveryContext;
        this.adapterName = adapterName;
        this.type = type;
        this.fulfillmentID = fulfillmentID;
        this.code = (type == COMPLETE || type == INVALID_FULFILLMENT_ID) ? 0: code;
        this.description = description;
    }

    public String getAdapterName() {
        return adapterName;
    }
    
    public int getCode() {
        return code;
    }

    public DeliveryContext getDeliveryContext(){
        return deliveryContext;
    }

    public String getDescription(){
        return description;
    }

    public String getFulfillmentID() {
        return fulfillmentID;
    }

    public int getType() {
        return type;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        final DeliveryEvent event = (DeliveryEvent) o;

        if (code != event.code) return false;
        if (type != event.type) return false;
        if (!adapterName.equals(event.adapterName)) return false;
        if (!deliveryContext.equals(event.deliveryContext)) return false;
        if (!description.equals(event.description)) return false;
        if (!fulfillmentID.equals(event.fulfillmentID)) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = adapterName.hashCode();
        result = 29 * result + code;
        result = 29 * result + deliveryContext.hashCode();
        result = 29 * result + description.hashCode();
        result = 29 * result + type;
        result = 29 * result + fulfillmentID.hashCode();
        return result;
    }

    public String toString() {
        return "Fulfillment ID = " + fulfillmentID + ", AdapterName = " + adapterName
                + ", Description = " + description + ", Code = " + code + ", Type = " + type;
    }

}